void getPots() // reading from 4051
{
  for (byte i = 0; i < 6; i++)
  {
    // set the selector pins HIGH and LOW to match the binary value of channel
    for(int bit = 0; bit < 3; bit++)
    {
      int pin = select[bit]; // the pin wired to the multiplexer select bit
      int isBitSet =  bitRead(i, bit); // true if given bit set in channel
      digitalWrite(pin, isBitSet);
    }
    //return map((constrain(analogRead(analogPin), 40, 1000)), 40, 1000, 0, 1023); // we're only using readings between 40 and 1000, as the extreme ends are unstable
    int reading = analogRead(analogPin);
    pot[i] = reading;
  }

  // see if it's time to lock the pots
  if (lockStep != 255 && currentStep == ((lockStep + 4) % 32))
    lockPot(6);
}

void potHandler()
{
  switch (mode)
  {
  case 0: // EDIT MODE
    potChecks();
    break;

  case 2: // TRIGGER MODE
    potChecks();
    break;
  }
}

void lockPot(byte Pot) // values of 0 - 5 for each pot and 6 for all
{
  if (Pot == 6)
  {
    for (byte i = 0; i < 6; i++)
      potLock[i] = pot[i];
  }
  else
    potLock[Pot] = pot[Pot];
  showingNumbers = false;
  lockStep = 255;
}

boolean unlockedPot(byte Pot) // check if a pot is locked or not
{
  if (potLock[Pot] == 9999)
    return true;
  else if (difference(potLock[Pot], pot[Pot]) > 35) // can lower the threshold value (35) for less jittery pots
  {
    for (byte i = 0; i < 6; i++)// lock all the other pots
    {
      if (i != Pot)
        lockPot(i);
    }
    potLock[Pot] = 9999;
    return true;
  }
  else
    return false;
}

// a handy DIFFERENCE FUNCTION
int difference(int i, int j)
{
  int k = i - j;
  if (k < 0)
    k = j - i;
  return k;
}

void potChecks()
{
  if (unlockedPot(0)) // hat pitch
  {
    if (pressed[39])
    {
      if (hatAuto[currentStep] != pot[0] >> 2)
      {
        hatAuto[currentStep] = pot[0] >> 2;
        hatAutoOn = true;
        hatPitch = hatAuto[currentStep];
        lockStep = currentStep;
      }
    }
    else
      if (hatPitch != pot[0] >> 2)
      {
        hatPitch = pot[0] >> 2;
        lockStep = currentStep;
        hatAutoOn = false;
      }
  }
  else
  {
    if (hatAutoOn)
      hatPitch = hatAuto[currentStep];
  }

  if (unlockedPot(1)) // crash pitch
  {
    if (pressed[39])
    {
      if (crashAuto[currentStep] != pot[1] >> 2)
      {
        crashAuto[currentStep] = pot[1] >> 2;
        crashAutoOn = true;
        crashPitch = crashAuto[currentStep];
        lockStep = currentStep;
      }
    }
    else
      if (crashPitch != pot[1] >> 2)
      {
        crashPitch = pot[1] >> 2;
        lockStep = currentStep;
        crashAutoOn = false;
      }
  }
  else
  {
    if (crashAutoOn)
      crashPitch = crashAuto[currentStep];
  }

  if (unlockedPot(2))
  {
    if (xorDistortion != pot[2] >> 4)
    {
      xorDistortion = pot[2] >> 4;
      lockStep = currentStep; 
    }
  }
  
  if (unlockedPot(3))
  {
    switch (currentVoice)
    {
    case 0: // kick
      if (kickVolume != pot[3] >> 2)
      {
        kickVolume = pot[3] >> 2;
        lockStep = currentStep;
        showingNumbers = true;
        number = kickVolume;
      } 
      break;
    case 1: // snare
      if (snareVolume != pot[3] >> 2)
      {
        snareVolume = pot[3] >> 2;
        lockStep = currentStep;
        showingNumbers = true;
        number = snareVolume;
      }
      break;
    case 2: // hat
      if (hatVolume != pot[3] >> 2)
      {
        hatVolume = pot[3] >> 2;
        lockStep = currentStep;
        showingNumbers = true;
        number = hatVolume;
      }
      break;
    case 3: // crash
      if (crashVolume != pot[3] >> 2)
      {
        crashVolume = pot[3] >> 2;
        lockStep = currentStep;
        showingNumbers = true;
        number = crashVolume;
      }
      break;

    }
  }

  if (unlockedPot(4))
  {
    if (tempSwing != pot[4] >> 7)
    {
      tempSwing = pot[4] >> 7;
      lockStep = currentStep;
      showingNumbers = true;
      number = tempSwing;
    }
  }

  if (unlockedPot(5))
  {
    if (pressed[39])
    {
      if (triggerChannel != map(pot[5], 0, 1023, 0, 17))
      {
        triggerChannel = constrain(map(pot[5], 0, 1023, 0, 17), 0, 16); // a weird line, but on my dodgy pots, it will give me a val of 15 as a maximum ;^/
        lockStep = currentStep;
        showingNumbers = true;
        number = triggerChannel;
      }
    }

    else if (bpm != map(pot[5], 0, 1023, 40, 255))
    {
      bpm = map(pot[5], 0, 1023, 40, 255);
      bpmChange();
    }
  }
}

void bpmChange()
{
  Timer1.setPeriod(60000000 / bpm / 96);
  lockStep = currentStep;
  showingNumbers = true;
  number = bpm;
}






















